/**
 * PETALS - PETALS Services Platform. Copyright (c) 2006 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: MailBCExternalListener.java 154 27 sept. 06 ofabre $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bc.mail.listeners;

import java.io.IOException;
import java.io.InputStream;
import java.util.Timer;
import java.util.logging.Level;

import javax.mail.Address;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Part;
import javax.mail.Flags.Flag;

import org.ow2.petals.bc.mail.Constants;
import org.ow2.petals.bc.mail.MailBCImpl;
import org.ow2.petals.component.framework.Constants.MEPConstants;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.listener.AbstractExternalListener;
import org.ow2.petals.component.framework.util.SourceHelper;

/**
 * This class is used to manage listeners for external mail addresses registered
 * during SU deployments
 * 
 * @author Adrien LOUIS - EBMWebSourcing
 * @author ofabre - EBM Websourcing
 * 
 */
public class MailBCExternalListener extends AbstractExternalListener {

    private Timer t;

    protected MailSessionManager mailSessionManager;

    protected SessionDescriptorBuilder sessionDescriptorBuilder;

    protected MimeMessageManager mimeMessageManager;

    protected ConfigurationExtensions extensions;

    @Override
    protected void init() {
        this.mailSessionManager = ((MailBCImpl) getComponent()).getMailSessionManager();
        this.sessionDescriptorBuilder = ((MailBCImpl) getComponent()).getSessionDescriptorBuilder();
        this.mimeMessageManager = ((MailBCImpl) getComponent()).getMimeMessageManager();
    }

    @Override
    protected void start() throws PEtALSCDKException {
        // Extract session information from SU extensions
        extensions = new ConfigurationExtensions(getConsumes().getAny());
        SessionDescriptor sessionDescriptor = sessionDescriptorBuilder.build(null, extensions);

        // Create and register the listener
        MailBCExternalScheduler listener = null;
        t = new Timer();
        listener = new MailBCExternalScheduler(getLogger(), this, sessionDescriptor,
                mailSessionManager);

        // Start listening for new mails
        t.schedule(listener, 0, Integer.parseInt(sessionDescriptor.getPeriod()));

    }

    @Override
    protected void stop() throws PEtALSCDKException {
        t.cancel();
    }

    /**
     * Send the message to the jbi service defined in the Consume node
     * 
     * @param message
     */
    protected void process(Message message) {

        getLogger().info("process an incoming email");

        Exchange exchange = null;

        try {
            exchange = this.createConsumeExchange(getConsumes(), MEPConstants.IN_ONLY_PATTERN);
            setJBIContentAndAttachments(message, exchange);
            setProperties(message, exchange);
        } catch (javax.jbi.messaging.MessagingException e) {
            getLogger().log(Level.SEVERE, "error with JBI Exchange creation", e);
        } catch (PEtALSCDKException e) {
            getLogger().log(Level.SEVERE, "error with XML Normalized message creation", e);
        } catch (MessagingException e) {
            getLogger().log(Level.SEVERE, "error with email access", e);
        } catch (IOException e) {
            getLogger().log(Level.SEVERE, "error with email access", e);
        }
        try {
            send(exchange);
        } catch (javax.jbi.messaging.MessagingException e) {
            getLogger().log(Level.SEVERE, "the JBI Exchange can not be sent", e);
            try {
                message.setFlag(Flag.DELETED, false);
            } catch (MessagingException e1) {
                getLogger().log(Level.SEVERE, "error with email access", e);
            }
        }
    }

    /**
     * set the body and attachments from the mail {@link Part} to the jbi
     * exchange. If the given message {@link Part} is a {@link Multipart}, all
     * sub part are recursively processed.
     * 
     * @param message
     *            the mail {@link Part} to process
     * @param exchange
     *            the resulting jbi exchange
     * @throws MessagingException
     * @throws IOException
     * @throws javax.jbi.messaging.MessagingException
     * @throws PEtALSCDKException
     */
    protected void setJBIContentAndAttachments(Part message, Exchange exchange) throws IOException,
            MessagingException, javax.jbi.messaging.MessagingException, PEtALSCDKException {
        Object content = message.getContent();
        if (content instanceof String) {
            if (exchange.getInMessageContentAsSource() == null) {
                getLogger().info("a text part is set to JBI Exchange content");
                exchange.setInMessageContent(SourceHelper.createSource("<body>" + content
                        + "</body>"));
            } else {
                getLogger().warning(
                        "a text part is found, but the JBI Exchange content has been already set");
            }
        } else if (content instanceof Multipart) {
            Multipart mp = (Multipart) content;
            int count = mp.getCount();
            for (int i = 0; i < count; i++) {
                setJBIContentAndAttachments(mp.getBodyPart(i), exchange);
            }
        } else if (content instanceof InputStream) {
            getLogger().info("add an attachment on JBI Exchange");
            exchange.addInMessageAttachment(message.getFileName(), message.getDataHandler());
        }
    }

    /**
     * Set ws addressing values : from, to, action(subject)
     * 
     * @param message
     * @param exchange
     * @throws MessagingException
     */
    protected void setProperties(Message message, Exchange exchange) throws MessagingException {
        String s_from = "";
        for (Address from : message.getFrom()) {
            s_from = s_from + from.toString() + ";";
        }
        s_from=s_from.substring(0, s_from.lastIndexOf(';'));
        
        String s_reply = "";
        for (Address reply : message.getReplyTo()) {
            s_reply = s_reply + reply.toString() + ";";
        }
        s_reply=s_reply.substring(0, s_reply.lastIndexOf(';'));
        
        String s_to = "";
        for (Address to : message.getAllRecipients()) {
            s_to = s_to + to.toString() + ";";
        }
        s_to=s_to.substring(0, s_to.lastIndexOf(';'));
        
        String s_subject = message.getSubject();

        exchange.getInMessage().setProperty(Constants.QNAME_WSA_FROM.toString(), s_from);
        getLogger().info("set property "+Constants.QNAME_WSA_FROM+" : "+s_from); 
        exchange.getInMessage().setProperty(Constants.QNAME_WSA_REPLY_TO.toString(), s_reply);
        getLogger().info("set property "+Constants.QNAME_WSA_REPLY_TO+" : "+s_reply); 
        exchange.getInMessage().setProperty(Constants.QNAME_WSA_ACTION.toString(), s_subject);
        getLogger().info("set property "+Constants.QNAME_WSA_ACTION+" : "+s_subject); 
    }
}
