/**
 * PETALS - PETALS Services Platform. Copyright (c) 2006 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: SessionDescriptorBuilder.java 154 27 sept. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.bc.mail.listeners;

import java.util.logging.Logger;

import javax.jbi.JBIException;

import org.ow2.petals.bc.mail.Constants;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.util.StringHelper;

/**
 * This class is used to extract information from an SMTP URI String
 * 
 * @author Adrien LOUIS - EBMWebSourcing
 * @author ofabre - EBM Websourcing
 * 
 */
public class SessionDescriptorBuilder {

    protected Logger log;

    /**
     * @param address
     *            address extracted from a "provides" node of the jbi
     *            descriptor.
     *            <p>
     *            Address conforms to the following pattern (a valid URI
     *            pattern):
     *            <p>
     * 
     *            
     *            smtp://<i>user</i>:<i>password</i>@<i>host</i>:<i>port</i>/from
     *            :<i>fromAddress</i>/to:<i>toAddress</i>.
     *            <p>
     *            <i>fromAddress</i> and <i>toAddress</i> are classical email
     *            addresses like : foo@bar.com.
     * @param log
     *            the component logger
     */
    public SessionDescriptorBuilder(Logger log) {
        super();
        this.log = log;
    }

    /**
     * Retrieve mail session properties from the given address and message
     * exchange.
     * 
     * Descriptor default values : <br/> hostname : the hostname, non null and
     * non empty<br/> port : the port, default value : "25"<br/> username : the
     * username, default value : no default<br/> password : the password,
     * default value : no default<br/> fromAddress : the return address, default
     * value : no default<br/> toAddress : the destination address, non null and
     * non empty<br/> period : time elapse between two new mail checkings,
     * default value : "60000" ms<br/> folder : the folder to check for new
     * mails, default value : "INBOX"<br/> scheme : the access or transport
     * protocol "smtp", "imap" or "pop3", non null and non empty<br/>
     * 
     * @param extensions
     *            extensions associated to the given message exchange
     * @param exchangeWrapper
     *            the message exchange wrapper used to extract an optionnal
     *            destination address
     * @return a Map of properties extracted from the given address.
     * @throws JBIException
     *             if address isn't a valid URI or if checking fails
     */
    public SessionDescriptor build(Exchange exchangeWrapper, ConfigurationExtensions extensions)
            throws PEtALSCDKException {
        SessionDescriptor sessionDescriptor = new SessionDescriptor();

        /*
         * Retrieve properties values from address URI
         */
        // Retrieve user and password
        sessionDescriptor.setUsername(extensions.get("user", null));
        sessionDescriptor.setPassword(extensions.get("password", null));

        // Retrieve protocol (smtp, imap, pop3)
        String scheme = extensions.get("scheme");
        sessionDescriptor.setScheme(scheme);

        // Retrieve host and port
        sessionDescriptor.setHostname(extensions.get("host"));

        if (Constants.MAIL_SCHEME_POP3.equals(scheme)) {
            sessionDescriptor.setPort(extensions.get("port", Constants.MAIL_POP3_PORT_DEFAULT));
        } else if (Constants.MAIL_SCHEME_IMAP.equals(scheme)) {
            sessionDescriptor.setPort(extensions.get("port", Constants.MAIL_IMAP_PORT_DEFAULT));
        } else {
            sessionDescriptor.setPort(extensions.get("port", Constants.MAIL_SMTP_PORT_DEFAULT));
        }

        // process Provides or Consumes values
        if (Constants.MAIL_SCHEME_SMTP.equalsIgnoreCase(scheme)) {

            // Retrieve "from" addresses
            String fromAddress = extensions.get(Constants.FROM_PATHELEMENT);
            // if the destination address is null, try to find it in the message
            // exchange, in case of an SMTP scheme
            if (StringHelper.isNullOrEmpty(fromAddress)) {
                if (exchangeWrapper != null) {
                    fromAddress = (String) exchangeWrapper.getInMessage().getProperty(
                            Constants.QNAME_WSA_FROM.toString());
                }
            }
            sessionDescriptor.setFromAddress(fromAddress);

            // Retrieve "replyTo" addresses
            String replyAddress = extensions.get(Constants.REPLY_PATHELEMENT);
            // if the destination address is null, try to find it in the message
            // exchange, in case of an SMTP scheme
            if (StringHelper.isNullOrEmpty(replyAddress)) {
                if (exchangeWrapper != null) {
                    replyAddress = (String) exchangeWrapper.getInMessage().getProperty(
                            Constants.QNAME_WSA_REPLY_TO.toString());
                }
            }
            sessionDescriptor.setReplyAddress(replyAddress);
         
            String toAddress = extensions.get(Constants.TO_PATHELEMENT);
            // Retrieve "to" addresses
            // if the destination address is null, try to find it in the message
            // exchange, in case of an SMTP scheme
            if (StringHelper.isNullOrEmpty(toAddress)) {
                if (exchangeWrapper != null) {
                    toAddress = (String) exchangeWrapper.getInMessage().getProperty(
                            Constants.QNAME_WSA_TO.toString());
                }
            }
            sessionDescriptor.setToAddress(toAddress);

            // Retrieve "subject"
            String subject = extensions.get(Constants.SUBJECT_PATHELEMENT);
            if (StringHelper.isNullOrEmpty(subject)) {
                if (exchangeWrapper != null) {
                    subject = (String) exchangeWrapper.getInMessage().getProperty(
                            Constants.QNAME_WSA_ACTION.toString());
                }
            }
            if (StringHelper.isNullOrEmpty(subject)) {
                subject = "petals-bc-mail";
            }
            sessionDescriptor.setSubject(subject);
        } else {
            // Retrieve period parameter ("checking for new mail" interval)
            sessionDescriptor.setPeriod(extensions.get(Constants.PERIOD_QUERYELEMENT,
                    Constants.PERIOD_DEFAULT));

            // Retrieve folder parameter (the folder to search for new mails)
            sessionDescriptor.setFolder(extensions.get(Constants.FOLDER_QUERYELEMENT,
                    Constants.FOLDER_DEFAULT));

            // Retrieve the Flag to set on read messages
            String s_expunge = extensions.get(Constants.EXPUNGE_PATHELEMENT);
            sessionDescriptor.setExpunge(Boolean.getBoolean(s_expunge));
        }
        // Check obligatory properties
        checkProperties(sessionDescriptor);

        return sessionDescriptor;
    }

    /**
     * Check the properties extracted from the address URI. Only hostname is
     * obligatory for all schemes. Only SMTP, POP3 and IMAP schemes are
     * supported. "toAddress" is need for SMTP scheme.
     * 
     * @param sessionDescriptor
     * @throws PEtALSCDKException
     *             if a checking step failed
     */
    protected void checkProperties(SessionDescriptor sessionDescriptor) throws PEtALSCDKException {
        String msg = "";
        if (StringHelper.isNullOrEmpty(sessionDescriptor.getHostname())) {
            msg = "The host name of the requested external " + "service must be specified";
            log.severe(msg);
            throw new PEtALSCDKException(msg);
        }
        if (!(Constants.MAIL_SCHEME_IMAP.equalsIgnoreCase(sessionDescriptor.getScheme())
                || Constants.MAIL_SCHEME_POP3.equalsIgnoreCase(sessionDescriptor.getScheme()) || Constants.MAIL_SCHEME_SMTP
                .equalsIgnoreCase(sessionDescriptor.getScheme()))) {
            msg = "The specified scheme isn't supported : " + sessionDescriptor.getScheme();
            log.severe(msg);
            throw new PEtALSCDKException(msg);
        }
    }
}
